function [frac_dim stats] = fracdim(x, y, index_renyi, firstpoint, lastpoint, plotflag)
% [frac_dim stats] = fracdim(x, y, index_renyi, firstpoint, lastpoint, plotflag)
% Calculates slope of a line to determine fractal dimension
%
% INPUT:  
%         x           cubes
%
%         y           if index_renyi = 0, y = log2 depending on cubes of the set
%                     if index_renyi = 1, y = sum(-p*log2(p)), where p -
%                     probability that point will be inside the cube
%
%         index_renyi calculate Renyi dimension with specified index (0 - Hausdorff or 1 - information)
%
%         firstpoint  firstpoint to lastpoint (including) points are used 
%         lastpoint   to calculate line slope the
%
%         plotflag    = 0 do not plot anything
%                     = 1 plot some stuff
%                     = 2 plot on previous
%
% OUTPUT: frac_dim    the estimated fractal dimension
%         stats       Errors in Least Squares approximation
%                     stats.ols_s - Least squares RMS error
%                     s.s - Robust RMS error
%
% (c) Ruslan Enikeev http://profitengines.ru

if nargin < 4
    firstpoint = -1;
end

if nargin < 5
    lastpoint = -1;
end

if nargin < 6
  plotflag = 0;
end

% To estimate the fractal dimension, the slope of the line 
% "closest" (in the robust least square sense) to the points (x,y) is
% calculated
if firstpoint <= 0
    firstpoint = 1;
end

if lastpoint <= 0 || lastpoint > length(y)
    lastpoint = length(y);
end

[param stats] = robustfit(x(firstpoint:lastpoint),y(firstpoint:lastpoint));

if index_renyi == 2
    frac_dim = param(2); % the estimated fractal dimension
else    
    frac_dim = -param(2); % the estimated fractal dimension
end

% plotting stuff
if (plotflag>0)
    if(plotflag==1)
      figure;
      plot(x, y, 'o', ...
           [x(1), x(end)], [param(2)*x(1) + param(1), param(2)*x(end) + param(1)], '-b');
      string = sprintf('Estimated fractal dimension = %.1f', frac_dim);
      title(string);
      xlabel('log2(\epsilon)');
      if index_renyi == 1
        ylabel('I(\epsilon)'); %'sum(-p*log2(p))'
      else
          if index_renyi == 0
            ylabel('log2 N(\epsilon)');
          else
            ylabel('log2 C(\epsilon)');
          end
      end
      grid on
    else
      str = get(get(gca,'Title'), 'String');
      str = {str; sprintf('Estimated fractal dimension (dotted) = %.1f', frac_dim)};
      
      title(str);
      
      hold on
      plot([x(1), x(end)], [param(2)*x(1) + param(1), param(2)*x(end) + param(1)], ':');
      hold off
    end
end